/*
  JWildfire - an image and animation processor written in Java 
  Copyright (C) 1995-2011 Andreas Maschke
  This is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser 
  General Public License as published by the Free Software Foundation; either version 2.1 of the 
  License, or (at your option) any later version.
 
  This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
  Lesser General Public License for more details.
  You should have received a copy of the GNU Lesser General Public License along with this software; 
  if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jwildfire.create.tina.variation;

import org.jwildfire.create.tina.base.XForm;
import org.jwildfire.create.tina.base.XYZPoint;
import org.jwildfire.base.Tools;
import static org.jwildfire.base.mathlib.Complex;
import static org.jwildfire.base.mathlib.MathLib.M_2_PI;


public class ExpSqrSqrtRecipAtanhFunc extends VariationFunc {
  private static final long serialVersionUID = 1L;
  
  private static final String PARAM_RE = "re";
  private static final String PARAM_IM = "im";
  private static final String PARAM_FLIP = "flip";
  
  private static final String[] paramNames = { PARAM_RE, PARAM_IM, PARAM_FLIP};
  private double re = 1.0;
  private double im = 0.0;
  private int flip = 0;
  
  @Override
  public void transform(FlameTransformationContext pContext, XForm pXForm, XYZPoint pAffineTP, XYZPoint pVarTP, double pAmount) {
  // atanh(1/sqrt(exp(z)/z2)^2 + 1)) {Until I think of a name...} by Whittaker Courtney 
 
   Complex z = new Complex(pAffineTP.x, pAffineTP.y);
   Complex z2 = new Complex( re, im);
  
       z.Exp(); 
       z.Div(z2);      
       z.Sqr();
   if ( flip == 1.0 ) {
       z.Flip();
       }    
       z.Inc();
       z.Sqrt();
       z.Recip();
       z.AtanH();

       z.Scale(pAmount * M_2_PI);
                                                                   
   if (pContext.random() < 0.5){                              
       pVarTP.x += z.re;  
       pVarTP.y += z.im;    
    }
   else {     
       pVarTP.x += -z.re; 
       pVarTP.y += -z.im;
    }                   
             
   if (pContext.isPreserveZCoordinate()) {
       pVarTP.z += pAmount * pAffineTP.z;
	}

  }
  
  @Override
  public String[] getParameterNames() {
    return paramNames;
  }

  @Override
  public Object[] getParameterValues() {
    return new Object[] {re, im, flip };
  }

  @Override
  public void setParameter(String pName, double pValue) {
    if (PARAM_RE.equalsIgnoreCase(pName))
      re = pValue;
    else if (PARAM_IM.equalsIgnoreCase(pName))
      im = pValue;
    else if (PARAM_FLIP.equalsIgnoreCase(pName))
      flip = (int) Tools.limitValue(pValue, 0, 1);
    else
      throw new IllegalArgumentException(pName);
  }
 

  @Override
  public String getName() {
    return "exp_sqr_sqrt_recip_atanh";
  }

}